@@ -213,7 +213,7 @@ G_STMT_START {                                                    \
 
 /* timestamp debugging macros */
 /**
- * GST_TIME_FORMAT:
+ * GST_TIME_FORMAT: (skip):
  *
  * A string that can be used in printf-like format strings to display a
  * #GstClockTime value in h:m:s format.  Use GST_TIME_ARGS() to construct
@@ -226,7 +226,7 @@ G_STMT_START {                                                    \
  */
 #define GST_TIME_FORMAT "u:%02u:%02u.%09u"
 /**
- * GST_TIME_ARGS:
+ * GST_TIME_ARGS: (skip):
  * @t: a #GstClockTime
  *
  * Format @t for the #GST_TIME_FORMAT format string. Note: @t will be
@@ -242,7 +242,7 @@ G_STMT_START {                                                    \
         GST_CLOCK_TIME_IS_VALID (t) ? \
         (guint) (((GstClockTime)(t)) % GST_SECOND) : 999999999
 /**
- * GST_STIME_FORMAT:
+ * GST_STIME_FORMAT: (skip):
  *
  * A string that can be used in printf-like format strings to display a signed
  * #GstClockTimeDiff or #gint64 value in h:m:s format.  Use GST_TIME_ARGS() to
@@ -257,7 +257,7 @@ G_STMT_START {                                                    \
  */
 #define GST_STIME_FORMAT "c%" GST_TIME_FORMAT
 /**
- * GST_STIME_ARGS:
+ * GST_STIME_ARGS: (skip):
  * @t: a #GstClockTimeDiff or #gint64
  *
  * Format @t for the #GST_STIME_FORMAT format string. Note: @t will be
@@ -339,13 +339,18 @@ typedef enum {
  * Cast to a clock entry
  */
 #define GST_CLOCK_ENTRY(entry)          ((GstClockEntry *)(entry))
+
+#ifndef GST_DISABLE_DEPRECATED
 /**
  * GST_CLOCK_ENTRY_CLOCK:
  * @entry: the entry to query
  *
  * Get the owner clock of the entry
+ *
+ * Deprecated: Use gst_clock_id_get_clock() instead.
  */
 #define GST_CLOCK_ENTRY_CLOCK(entry)    ((entry)->clock)
+#endif
 /**
  * GST_CLOCK_ENTRY_TYPE:
  * @entry: the entry to query
@@ -387,7 +392,13 @@ typedef enum {
 struct _GstClockEntry {
   gint                  refcount;
   /*< protected >*/
+#ifndef GST_REMOVE_DEPRECATED
+#ifndef GST_DISABLE_DEPRECATED
   GstClock              *clock;
+#else
+  gpointer               _clock;
+#endif
+#endif
   GstClockEntryType      type;
   GstClockTime           time;
   GstClockTime           interval;
@@ -413,7 +424,7 @@ struct _GstClockEntry {
  * @GST_CLOCK_FLAG_CAN_SET_RESOLUTION: clock's resolution can be changed
  * @GST_CLOCK_FLAG_CAN_SET_MASTER: clock can be slaved to a master clock
  * @GST_CLOCK_FLAG_NEEDS_STARTUP_SYNC: clock needs to be synced before it can be used
- *     (Since 1.6)
+ *     (Since: 1.6)
  * @GST_CLOCK_FLAG_LAST: subclasses can add additional flags starting from this flag
  *
  * The capabilities of this clock
@@ -600,6 +611,12 @@ void                    gst_clock_id_unref              (GstClockID id);
 GST_API
 gint                    gst_clock_id_compare_func       (gconstpointer id1, gconstpointer id2);
 
+GST_API
+GstClock *              gst_clock_id_get_clock          (GstClockID id);
+
+GST_API
+gboolean                gst_clock_id_uses_clock         (GstClockID id, GstClock * clock);
+
 GST_API
 GstClockTime            gst_clock_id_get_time           (GstClockID id);
 
