@@ -18,6 +18,7 @@
  */
 /**
  * SECTION:gstnettimeprovider
+ * @title: GstNetTimeProvider
  * @short_description: Special object that exposed the time of a clock
  *                     on the network.
  * @see_also: #GstClock, #GstNetClientClock, #GstPipeline
@@ -39,12 +40,14 @@
 
 #include "gstnettimeprovider.h"
 #include "gstnettimepacket.h"
+#include "gstnetutils.h"
 
 GST_DEBUG_CATEGORY_STATIC (ntp_debug);
 #define GST_CAT_DEFAULT (ntp_debug)
 
 #define DEFAULT_ADDRESS         "0.0.0.0"
 #define DEFAULT_PORT            5637
+#define DEFAULT_QOS_DSCP        -1
 
 #define IS_ACTIVE(self) (g_atomic_int_get (&((self)->priv->active)))
 
@@ -54,16 +57,15 @@ enum
   PROP_PORT,
   PROP_ADDRESS,
   PROP_CLOCK,
-  PROP_ACTIVE
+  PROP_ACTIVE,
+  PROP_QOS_DSCP
 };
 
-#define GST_NET_TIME_PROVIDER_GET_PRIVATE(obj)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_NET_TIME_PROVIDER, GstNetTimeProviderPrivate))
-
 struct _GstNetTimeProviderPrivate
 {
   gchar *address;
   int port;
+  gint qos_dscp;                /* ATOMIC */
 
   GThread *thread;
 
@@ -96,7 +98,7 @@ static void gst_net_time_provider_get_property (GObject * object, guint prop_id,
 
 #define gst_net_time_provider_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstNetTimeProvider, gst_net_time_provider,
-    GST_TYPE_OBJECT, _do_init);
+    GST_TYPE_OBJECT, G_ADD_PRIVATE (GstNetTimeProvider) _do_init);
 
 static void
 gst_net_time_provider_class_init (GstNetTimeProviderClass * klass)
@@ -107,8 +109,6 @@ gst_net_time_provider_class_init (GstNetTimeProviderClass * klass)
 
   g_assert (sizeof (GstClockTime) == 8);
 
-  g_type_class_add_private (klass, sizeof (GstNetTimeProviderPrivate));
-
   gobject_class->finalize = gst_net_time_provider_finalize;
   gobject_class->set_property = gst_net_time_provider_set_property;
   gobject_class->get_property = gst_net_time_provider_get_property;
@@ -130,15 +130,21 @@ gst_net_time_provider_class_init (GstNetTimeProviderClass * klass)
       g_param_spec_boolean ("active", "Active",
           "TRUE if the clock will respond to queries over the network", TRUE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_QOS_DSCP,
+      g_param_spec_int ("qos-dscp", "QoS diff srv code point",
+          "Quality of Service, differentiated services code point (-1 default)",
+          -1, 63, DEFAULT_QOS_DSCP,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 gst_net_time_provider_init (GstNetTimeProvider * self)
 {
-  self->priv = GST_NET_TIME_PROVIDER_GET_PRIVATE (self);
+  self->priv = gst_net_time_provider_get_instance_private (self);
 
   self->priv->port = DEFAULT_PORT;
   self->priv->address = g_strdup (DEFAULT_ADDRESS);
+  self->priv->qos_dscp = DEFAULT_QOS_DSCP;
   self->priv->thread = NULL;
   self->priv->active = TRUE;
 }
@@ -171,12 +177,15 @@ gst_net_time_provider_thread (gpointer data)
   GSocket *socket = self->priv->socket;
   GstNetTimePacket *packet;
   GError *err = NULL;
+  gint cur_qos_dscp = DEFAULT_QOS_DSCP;
+  gint new_qos_dscp;
 
   GST_INFO_OBJECT (self, "time provider thread is running");
 
   while (TRUE) {
     GSocketAddress *sender_addr = NULL;
 
+
     GST_LOG_OBJECT (self, "waiting on socket");
     if (!g_socket_condition_wait (socket, G_IO_IN, cancel, &err)) {
       GST_INFO_OBJECT (self, "socket error: %s", err->message);
@@ -202,6 +211,14 @@ gst_net_time_provider_thread (gpointer data)
       continue;
     }
 
+    /* before next sending check if need to change QoS */
+    new_qos_dscp = self->priv->qos_dscp;
+    if (cur_qos_dscp != new_qos_dscp &&
+        gst_net_utils_set_socket_dscp (socket, new_qos_dscp)) {
+      GST_DEBUG_OBJECT (self, "changed QoS DSCP to: %d", new_qos_dscp);
+      cur_qos_dscp = new_qos_dscp;
+    }
+
     if (IS_ACTIVE (self)) {
       /* do what we were asked to and send the packet back */
       packet->remote_time = gst_clock_get_time (self->priv->clock);
@@ -244,6 +261,9 @@ gst_net_time_provider_set_property (GObject * object, guint prop_id,
     case PROP_ACTIVE:
       g_atomic_int_set (&self->priv->active, g_value_get_boolean (value));
       break;
+    case PROP_QOS_DSCP:
+      g_atomic_int_set (&self->priv->qos_dscp, g_value_get_int (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -269,6 +289,9 @@ gst_net_time_provider_get_property (GObject * object, guint prop_id,
     case PROP_ACTIVE:
       g_value_set_boolean (value, IS_ACTIVE (self));
       break;
+    case PROP_QOS_DSCP:
+      g_value_set_int (value, self->priv->qos_dscp);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -433,7 +456,7 @@ gst_net_time_provider_initable_iface_init (gpointer g_iface)
  *
  * Allows network clients to get the current time of @clock.
  *
- * Returns: the new #GstNetTimeProvider, or NULL on error
+ * Returns: (transfer full): the new #GstNetTimeProvider, or NULL on error
  */
 GstNetTimeProvider *
 gst_net_time_provider_new (GstClock * clock, const gchar * address, gint port)
@@ -447,5 +470,8 @@ gst_net_time_provider_new (GstClock * clock, const gchar * address, gint port)
       g_initable_new (GST_TYPE_NET_TIME_PROVIDER, NULL, NULL, "clock", clock,
       "address", address, "port", port, NULL);
 
+  /* Clear floating flag */
+  g_object_ref_sink (ret);
+
   return ret;
 }
