@@ -852,7 +852,7 @@ gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_boolean (value, identity->sync);
       break;
     case PROP_TS_OFFSET:
-      identity->ts_offset = g_value_get_int64 (value);
+      g_value_set_int64 (value, identity->ts_offset);
       break;
     case PROP_CHECK_IMPERFECT_TIMESTAMP:
       g_value_set_boolean (value, identity->check_imperfect_timestamp);
@@ -976,8 +976,12 @@ gst_identity_query (GstBaseTransform * base, GstPadDirection direction,
     GST_OBJECT_LOCK (identity);
     if (live)
       identity->upstream_latency = min;
-    else
+    else {
       identity->upstream_latency = 0;
+      /* if we interface a non-live source, then we know there is no
+       * limit in the maximum latency */
+      max = -1;
+    }
     GST_OBJECT_UNLOCK (identity);
 
     gst_query_set_latency (query, live || identity->sync, min, max);
